<?php

namespace Drupal\address_js_geocoder\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\address\Plugin\Field\FieldWidget\AddressDefaultWidget;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\address_js_geocoder\Ajax\GeocodeAddressCommand;

/**
 * Plugin implementation of the 'address_geocoder' widget.
 *
 * @FieldWidget(
 *   id = "address_geocoder",
 *   module = "address_js_geocoder",
 *   label = @Translation("Address geocoder"),
 *   field_types = {
 *     "address"
 *   }
 * )
 */
class AddressGeocoder extends AddressDefaultWidget implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'geofield_target' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [] + parent::settingsForm($form, $form_state);

    $elements['geofield_target'] = [
      '#type' => 'textfield',
      '#title' => t('Geofield target'),
      '#default_value' => $this->getSetting('geofield_target'),
      '#description' => t('The geofield field machine name that change this address will update.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [] + parent::settingsSummary();

    if (!empty($this->getSetting('geofield_target'))) {
      $summary[] = t('Changing this address will update: @geofield', ['@geofield' => $this->getSetting('geofield_target')]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    //'#default_value' => isset($items[$delta]->value) ? $items[$delta]->value : NULL,

    $field_name = $this->fieldDefinition->getName();

    $form['#attached']['drupalSettings']['address_selector'] = 'edit-' . str_replace('_', "-", $field_name ) . "-" . $delta;
    $form['#attached']['library'][] = 'address_js_geocoder/address_js_geocoder-library';

    $element['autogeocode'] = [
      '#type' => 'checkbox',
      '#title' => $this->
      t('Automatically position on map'),
      '#default_value' => TRUE,
    ];


    $element['request_geocode'] = array(
      '#type' => 'button',
      '#value' => t('Resolve in map'),
      '#ajax' => array(
        'callback' => [$this, 'requestGeocodeCallback'],
        'wrapper' => 'geocoded-select-wrapper', // This element is updated with this AJAX callback.
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Requesting geocode...'),
        ],
      )
    );

    $element['geocoded_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Resolved addresses options'),
      '#description' => $this->t("This option will be positioned at map below"),
      '#empty_value' => '',
      '#empty_option' => '- Select a value -',
      '#default_value' => '',
      '#prefix' => '<div id="geocoded-select-wrapper">',
      '#suffix' => '</div>',
      '#validated' => TRUE,
      '#options' => [
      ],
      '#ajax' => [
        'callback' => [$this, 'geocodedSelectedCallback'], //alternative notation
        //'disable-refocus' => FALSE, // Or TRUE to prevent re-focusing on the triggering element.
        //'event' => 'change',
        'wrapper' => 'edit-output', // This element is updated with this AJAX callback.
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Showing in map...'),
        ],
      ],
    ];

    // Create a textbox that will be updatem
    // when the user selects an item from the select box above.
    $element['container']['resolved_address'] = [
      '#type' => 'hidden',
      '#size' => '60',
      '#disabled' => TRUE,
      '#value' => '',
    ];

    return $element;
  }

  /**
   *  TODO WHENEVER it is implemented the autosubmit of address via fapi not
   *  jquery.
   *  
   *  Adds ajax callback to all address fields.
   */
  public function addressMapTriggering(array $element, FormStateInterface &$form_state, $form){
    // TODO WHENEVER it is implemented the autosubmit of address via FAPI not
    // jquery;
    $trigger_fields = ['address_line1', /*'address_line2',*/ 'postal_code', 'locality', 'country'];
    foreach ($trigger_fields as $trigger_field){
      $element[$trigger_field]['#ajax'] = [
        'callback' => [ 'Drupal\address_js_geocoder\Plugin\Field\FieldWidget\AddressGeocoder', 'autocheckCallback'],
      ];
    }
    return $element;
  }

  /**
   *  TODO WHENEVER it is implemented the autosubmit of address via fapi not
   *  jquery.
   */
  public function autocheckCallback(array &$form, FormStateInterface $form_state) {
      // It must fill the hidden field after that wait a timeout and
      // submit value to be geocoded... But address is multifield and so
      // on it's not easy to bind an event ti this callback.
    $field_name = $this->fieldDefinition->getName();
    $response = new AjaxResponse();
    $address = $form_state->getValue($field_name);

    $address_string = $this->addressToString($address);
    $form[$field_name]["widget"][0]['container']["resolved_address"]["value"] = $address_string;
    //TODO BELOW pass to data drupal selector  $form[$field_name]['#attributes']['data-drupal-selector'];
    $response->addCommand(new ReplaceCommand("#resolved_address-output", $form[$field_name]["widget"][0]['resolved_address'] ));
    return $response;

  }

/**
 * Get the value from geocoded_select field and fill geofield target with the 
 * selected coords.
 *
 */
public function geocodedSelectedCallback(array &$form, FormStateInterface $form_state) {
  $response = new AjaxResponse();
  $field_name = $this->fieldDefinition->getName();

  if ($selectedValue = $form_state->getValues()[$field_name][0]['geocoded_select'] ){
    $coords_array = explode("|", $selectedValue);
    $coords = [
      "latitude" => $coords_array[0],
      "longitude" => $coords_array[1]
    ];
    $address_widget_selector = $form[$field_name]['#attributes']['data-drupal-selector'];
    $target_geofield_selector = $form[$this->getSetting('geofield_target')]['#attributes']['data-drupal-selector'];
    $response->addCommand(new GeocodeAddressCommand([
      "widget" => $address_widget_selector,
      "field_name" => $field_name
    ], $coords, [
      "widget" => $target_geofield_selector,
      "field_name" => $this->getSetting('geofield_target')
    ]));
  }
  return $response;

}

/**
 * An Ajax callback to request the geocode via geocode api.
 */
public function requestGeocodeCallback(array &$form, FormStateInterface $form_state) {
  $field_name = $this->fieldDefinition->getName();
  // TODO Check if geofield_target exists.
  $entity = $form_state->getFormObject()->getEntity();
  if (!$entity->hasField($this->getSetting('geofield_target'))){
    $form_state->setErrorByName($field_name, $this->t("The target @field field doesn't exist.",['@field' => $field_name]));
    $form[$field_name]["widget"][0]['geocoded_select']['#options'] = [];
    return  $form[$field_name]["widget"][0]['geocoded_select'];
  }
  $target_field = $form_state->getFormObject()->getEntity()->getFields()[$this->getSetting('geofield_target')]->getFieldDefinition();
  // Check if target field has geocode enabled
  /** @var \Drupal\geocoder_field\GeocoderFieldPluginInterface $field_plugin */ //TODO DI!
  if (!$target_field_plugin = \Drupal::service('geocoder_field.plugin.manager.field')->getPluginByFieldType($target_field->getType())) {
    // There's no geocoding field plugin to handle this type of field.
    return;
  }
  $target_field_config = $target_field->getThirdPartySettings('geocoder_field');
  $providers_ids = $target_field->getThirdPartySettings('geocoder_field')["providers"];
  $providers = \Drupal::entityTypeManager()->getStorage('geocoder_provider')->loadMultiple($providers_ids);


  $parents = array_merge($form['#parents'], [$field_name]);
  $submitted_values = $form_state->getValue($field_name);
  // Prepare our textfield. check if the example select field has a selected option.
  if ($address = $form_state->getValue($field_name)) {
    $form_root = $form[$field_name]["widget"][0];
    // Now get the address from $this
    $address = $form_state->getValue($field_name);

    $address_string = $this->addressToString($address);
    // Geocode the address:
    $addressCollection = \Drupal::service('geocoder')->geocode($address_string, $providers);

    // Set select options from results!!!
    $geocode_opts = $addressCollection;
    $form[$field_name]["widget"][0]['geocoded_select']['#options'] = [];
    foreach($geocode_opts as $key => $val){
      $form[$field_name]["widget"][0]['geocoded_select']['#options'][$val->getCoordinates()->getLatitude() ."|". $val->getCoordinates()->getLongitude()] = $val->getFormattedAddress();
    }
    // Place the text of the selected option in our textfield.
    $form[$field_name]["widget"][0]['container']['output']['#value'] = "aaa";//print_r($selectedValue, TRUE);
  }

  $response = new AjaxResponse();
  // return commands Replace the options from search
  // and trigger select change event to populate the map
  $response->addCommand(new ReplaceCommand("#geocoded-select-wrapper", $form[$field_name]["widget"][0]['geocoded_select'] ));
  $response->addCommand(new InvokeCommand('#geocoded-select-wrapper select', 'trigger', ['change']));

  return $response;
}

/**
 * Sets the address from Adress element type to string
 *
 * @param array $address_fields
 *   The address fields from address->value
 */
protected function addressToString(array $address_fields){

  $addresses = [];
  foreach($address_fields as $delta => $item){
      $address[] = !empty($item["address"]['address_line1']) ? $item["address"]['address_line1'] : NULL;
      $address[] = !empty($item["address"]['address_line2']) ? $item["address"]['address_line2'] : NULL;
      $address[] = !empty($item["address"]['postal_code']) ? $item["address"]['postal_code'] : NULL;
      $address[] = !empty($item["address"]['locality']) ? $item["address"]['locality'] : NULL;
      $address[] = !empty($item["address"]['country']) ? $item["address"]['country'] : NULL;
      $addresses[] = implode(' ', array_filter($address));
  }
  //TODO allow multiple address lookup By now only first...
  return $addresses[0];

}

}
